"use strict";
var util = require("util");
var path = require("path");
var helpers = require("./helpers");
function Exception() {
}
Exception.prototype = new Error();
function resolveCallStack(error) {
    var stackLines = error.stack.split("\n");
    var parsed = _.map(stackLines, function (line) {
        var match = line.match(/^\s*at ([^(]*) \((.*?):([0-9]+):([0-9]+)\)$/);
        if (match) {
            return match;
        }
        match = line.match(/^\s*at (.*?):([0-9]+):([0-9]+)$/);
        if (match) {
            match.splice(1, 0, "<anonymous>");
            return match;
        }
        return line;
    });
    var SourceMapConsumer = require("./vendor/source-map").sourceMap.SourceMapConsumer;
    var fs = require("fs");
    var remapped = _.map(parsed, function (parsedLine) {
        if (_.isString(parsedLine)) {
            return parsedLine;
        }
        var functionName = parsedLine[1];
        var fileName = parsedLine[2];
        var line = +parsedLine[3];
        var column = +parsedLine[4];
        var mapFileName = fileName + ".map";
        if (!fs.existsSync(mapFileName)) {
            return parsedLine.input;
        }
        var mapData = JSON.parse(fs.readFileSync(mapFileName).toString());
        var consumer = new SourceMapConsumer(mapData);
        var sourcePos = consumer.originalPositionFor({ line: line, column: column });
        var source = path.join(path.dirname(fileName), sourcePos.source);
        return util.format("    at %s (%s:%s:%s)", functionName, source, sourcePos.line, sourcePos.column);
    });
    var outputMessage = remapped.join("\n");
    if (outputMessage.indexOf(error.message) === -1) {
        outputMessage = outputMessage.replace(/Error/, "Error: " + error.message);
    }
    return outputMessage;
}
function installUncaughtExceptionListener() {
    process.on("uncaughtException", function (err) {
        var callstack = err.stack;
        if (callstack) {
            callstack = resolveCallStack(err);
        }
        console.log(callstack || err.toString());
        if (!$injector.resolve("staticConfig").disableAnalytics) {
            try {
                var analyticsService = $injector.resolve("analyticsService");
                analyticsService.trackException(err, callstack);
            }
            catch (e) {
                console.log("Error while reporting exception: " + e);
            }
        }
        process.exit(127 /* UNKNOWN */);
    });
}
exports.installUncaughtExceptionListener = installUncaughtExceptionListener;
var Errors = (function () {
    function Errors() {
        this.printCallStack = false;
    }
    Errors.prototype.fail = function (optsOrFormatStr) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var opts = optsOrFormatStr;
        if (_.isString(opts)) {
            opts = { formatStr: opts };
        }
        args.unshift(opts.formatStr);
        var exception = new Exception();
        exception.name = opts.name || "Exception";
        exception.message = util.format.apply(null, args);
        exception.stack = new Error(exception.message).stack;
        exception.errorCode = opts.errorCode || 127 /* UNKNOWN */;
        exception.suppressCommandHelp = opts.suppressCommandHelp;
        throw exception;
    };
    Errors.prototype.failWithoutHelp = function (message) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        args.unshift(message);
        this.fail({ formatStr: util.format.apply(null, args), suppressCommandHelp: true });
    };
    Errors.prototype.beginCommand = function (action, printCommandHelp) {
        var _this = this;
        return (function () {
            try {
                return action().wait();
            }
            catch (ex) {
                console.log(_this.printCallStack ? resolveCallStack(ex) : "\x1B[31;1m" + ex.message + "\x1B[0m");
                if (!ex.suppressCommandHelp) {
                    printCommandHelp().wait();
                }
                process.exit(_.isNumber(ex.errorCode) ? ex.errorCode : 127 /* UNKNOWN */);
            }
        }).future()();
    };
    Errors.prototype.executeAction = function (action) {
        try {
            return action();
        }
        catch (ex) {
            console.log(this.printCallStack ? resolveCallStack(ex) : "\x1B[31;1m" + ex.message + "\x1B[0m");
            process.exit(_.isNumber(ex.errorCode) ? ex.errorCode : 127 /* UNKNOWN */);
        }
    };
    Errors.prototype.verifyHeap = function (message) {
        if (global.gc) {
            console.log("verifyHeap: '%s'", message);
            global.gc();
        }
    };
    Errors.prototype.getParsedOptions = function (options, shorthands, clientName) {
        var _this = this;
        var action = function () {
            var yargs = require("yargs");
            _.each(options, function (type, opt) {
                if (type === String) {
                    yargs.string(opt);
                }
                else if (type === Boolean) {
                    yargs.boolean(opt);
                }
            });
            Object.keys(shorthands).forEach(function (key) { return yargs.alias(key, shorthands[key]); });
            var argv = yargs.argv;
            var parsed = {};
            _.each(_.keys(argv), function (opt) { return parsed[opt] = (typeof argv[opt] === "number") ? argv[opt].toString() : argv[opt]; });
            _this.validateYargsArguments(parsed, options, shorthands, clientName);
            return parsed;
        };
        return this.executeAction(action);
    };
    Errors.prototype.getYargsOriginalOption = function (option) {
        var matchUpperCaseLetters = option.match(/(.+?)([A-Z])(.*)/);
        if (matchUpperCaseLetters) {
            var secondaryPresentation = util.format("%s-%s%s", matchUpperCaseLetters[1], matchUpperCaseLetters[2].toLowerCase(), matchUpperCaseLetters[3] || '');
            return this.getYargsOriginalOption(secondaryPresentation);
        }
        return option;
    };
    Errors.prototype.validateYargsArguments = function (parsed, knownOpts, shorthands, clientName) {
        var _this = this;
        var knownOptionsKeys = _.keys(knownOpts);
        _.each(_.keys(parsed), function (opt) {
            var option = shorthands[opt] || opt;
            var secondaryPresentation = _this.getYargsOriginalOption(option);
            option = _.contains(knownOptionsKeys, secondaryPresentation) ? secondaryPresentation : option;
            if (option !== "_" && option !== "$0" && !knownOpts[option]) {
                _this.failWithoutHelp("The option '%s' is not supported. To see command's options, use '$ %s help %s'. To see all commands use '$ %s help'.", opt, clientName, process.argv[2], clientName);
            }
            else if (knownOpts[option] !== Boolean && typeof (parsed[opt]) === 'boolean') {
                _this.failWithoutHelp("The option '%s' requires a value.", opt);
            }
            else if (opt !== "_" && _.isArray(parsed[opt])) {
                _this.failWithoutHelp("You have set the %s option multiple times. Check the correct command syntax below and try again.", opt);
            }
            else if (knownOpts[option] === String && helpers.isNullOrWhitespace(parsed[opt])) {
                _this.failWithoutHelp("The option '%s' requires non-empty value.", opt);
            }
            else if (knownOpts[option] === Boolean && typeof (parsed[opt]) !== 'boolean') {
                _this.failWithoutHelp("The option '%s' does not accept values.", opt);
            }
        });
    };
    Errors.prototype.validateArgs = function (client, knownOpts, shorthands) {
        return this.getParsedOptions(knownOpts, shorthands, client);
    };
    return Errors;
})();
exports.Errors = Errors;
$injector.register("errors", Errors);
