"use strict";
var util = require("util");
var path = require("path");
var temp = require("temp");
var helpers = require("../../helpers");
var os = require("os");
var options = require("./../../options");
var LiveSyncCommands = (function () {
    function LiveSyncCommands() {
    }
    LiveSyncCommands.DeployProjectCommand = function (liveSyncUrl) {
        return util.format("DeployProject %s \r", liveSyncUrl);
    };
    LiveSyncCommands.ReloadStartViewCommand = function () {
        return "ReloadStartView \r";
    };
    LiveSyncCommands.SyncFilesCommand = function () {
        return "SyncFiles \r";
    };
    LiveSyncCommands.RefreshCurrentViewCommand = function () {
        return "RefreshCurrentView \r";
    };
    return LiveSyncCommands;
})();
var AndroidDevice = (function () {
    function AndroidDevice(identifier, adb, $logger, $fs, $childProcess, $errors, $staticConfig, $opener, $devicePlatformsConstants, $logcatHelper) {
        this.identifier = identifier;
        this.adb = adb;
        this.$logger = $logger;
        this.$fs = $fs;
        this.$childProcess = $childProcess;
        this.$errors = $errors;
        this.$staticConfig = $staticConfig;
        this.$opener = $opener;
        this.$devicePlatformsConstants = $devicePlatformsConstants;
        this.$logcatHelper = $logcatHelper;
        this._tmpRoots = {};
        this.defaultNodeInspectorUrl = "http://127.0.0.1:8080/debug";
        var details = this.getDeviceDetails().wait();
        this.model = details.model;
        this.name = details.name;
        this.version = details.release;
        this.vendor = details.brand;
    }
    AndroidDevice.prototype.getDeviceDetails = function () {
        var _this = this;
        return (function () {
            var requestDeviceDetailsCommand = _this.composeCommand("shell cat /system/build.prop");
            var details = _this.$childProcess.exec(requestDeviceDetailsCommand).wait();
            var parsedDetails = {};
            details.split(/\r?\n|\r/).forEach(function (value) {
                var match = /(?:ro\.build\.version|ro\.product)\.(.+)=(.+)/.exec(value);
                if (match) {
                    parsedDetails[match[1]] = match[2];
                }
            });
            return parsedDetails;
        }).future()();
    };
    AndroidDevice.prototype.getPlatform = function () {
        return this.$devicePlatformsConstants.Android;
    };
    AndroidDevice.prototype.getIdentifier = function () {
        return this.identifier;
    };
    AndroidDevice.prototype.getDisplayName = function () {
        return this.name;
    };
    AndroidDevice.prototype.getModel = function () {
        return this.model;
    };
    AndroidDevice.prototype.getVersion = function () {
        return this.version;
    };
    AndroidDevice.prototype.getVendor = function () {
        return this.vendor;
    };
    AndroidDevice.prototype.getInstalledApplications = function () {
        var _this = this;
        return (function () {
            if (!_this._installedApplications) {
                var listPackagesCommand = _this.composeCommand("shell pm list packages");
                var result = _this.$childProcess.exec(listPackagesCommand).wait();
                _this._installedApplications = _.map(result.split(os.EOL), function (packageString) {
                    var match = packageString.match(/package:(.+)/);
                    return match ? match[1] : null;
                }).filter(function (parsedPackage) { return parsedPackage != null; });
            }
            return _this._installedApplications;
        }).future()();
    };
    AndroidDevice.prototype.sendBroadcastToDevice = function (action, extras) {
        var _this = this;
        if (extras === void 0) { extras = {}; }
        return (function () {
            var broadcastCommand = _this.composeCommand("shell am broadcast -a \"%s\"", action);
            _.each(extras, function (value, key) { return broadcastCommand += util.format(" -e \"%s\" \"%s\"", key, value); });
            var result = _this.$childProcess.exec(broadcastCommand).wait();
            var match = result.match(/Broadcast completed: result=(\d+)/);
            if (match) {
                return +match[1];
            }
            _this.$errors.fail("Unable to broadcast to android device:\n%s", result);
        }).future()();
    };
    AndroidDevice.prototype.getLiveSyncRoot = function (appIdentifier, liveSyncVersion) {
        if (!this._tmpRoots[appIdentifier.appIdentifier]) {
            if (liveSyncVersion === 2) {
                this._tmpRoots[appIdentifier.appIdentifier] = util.format(AndroidDevice.DEVICE_TMP_DIR_FORMAT_V2, appIdentifier.appIdentifier);
            }
            else if (liveSyncVersion === 3) {
                this._tmpRoots[appIdentifier.appIdentifier] = util.format(AndroidDevice.DEVICE_TMP_DIR_FORMAT_V3, appIdentifier.appIdentifier);
            }
            else {
                this.$errors.fail("Unsupported LiveSync version: %d", liveSyncVersion);
            }
        }
        return this._tmpRoots[appIdentifier.appIdentifier];
    };
    AndroidDevice.prototype.composeCommand = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i - 0] = arguments[_i];
        }
        var command = util.format.apply(null, args);
        var result = util.format("\"%s\" -s %s", this.adb, this.identifier);
        if (!helpers.isNullOrWhitespace(command)) {
            result += util.format(" %s", command);
        }
        return result;
    };
    AndroidDevice.prototype.startPackageOnDevice = function (packageName) {
        var _this = this;
        return (function () {
            var startPackageCommand = _this.composeCommand("shell am start -a android.intent.action.MAIN -n %s/%s -c android.intent.category.LAUNCHER", packageName, _this.$staticConfig.START_PACKAGE_ACTIVITY_NAME);
            _this.$childProcess.exec(startPackageCommand).wait();
            if (!options.justlaunch) {
                _this.openDeviceLogStream();
            }
        }).future()();
    };
    AndroidDevice.prototype.deploy = function (packageFile, packageName) {
        var _this = this;
        return (function () {
            var uninstallCommand = _this.composeCommand("shell pm uninstall \"%s\"", packageName);
            _this.$childProcess.exec(uninstallCommand).wait();
            var installCommand = _this.composeCommand("install -r \"%s\"", packageFile);
            _this.$childProcess.exec(installCommand).wait();
            _this.startPackageOnDevice(packageName).wait();
            _this.$logger.info("Successfully deployed on device with identifier '%s'", _this.getIdentifier());
        }).future()();
    };
    AndroidDevice.prototype.tcpForward = function (src, dest) {
        var tcpForwardCommand = this.composeCommand("forward tcp:%d tcp:%d", src.toString(), dest.toString());
        this.$childProcess.exec(tcpForwardCommand).wait();
    };
    AndroidDevice.prototype.startDebuggerClient = function (port) {
        var _this = this;
        return (function () {
            var nodeInspectorModuleFilePath = require.resolve("node-inspector");
            var nodeInspectorModuleDir = path.dirname(nodeInspectorModuleFilePath);
            var nodeInspectorFullPath = path.join(nodeInspectorModuleDir, "bin", "inspector");
            _this.$childProcess.spawn(process.argv[0], [nodeInspectorFullPath, "--debug-port", port.toString()], { stdio: "ignore", detached: true });
        }).future()();
    };
    AndroidDevice.prototype.openDebuggerClient = function (url) {
        this.$opener.open(url, "chrome");
    };
    AndroidDevice.prototype.printDebugPort = function (packageName) {
        var res = this.$childProcess.spawnFromEvent(this.adb, ["shell", "am", "broadcast", "-a", packageName + "-GetDgbPort"], "exit").wait();
        this.$logger.info(res.stdout);
    };
    AndroidDevice.prototype.attachDebugger = function (packageName) {
        var startDebuggerCommand = this.composeCommand("shell am broadcast -a \"%s-Debug\" --ez enable true", packageName);
        var port = options.debugPort;
        if (port > 0) {
            startDebuggerCommand += " --ei debuggerPort " + options["debug-port"];
            this.$childProcess.exec(startDebuggerCommand).wait();
        }
        else {
            var res = this.$childProcess.spawnFromEvent(this.adb, ["shell", "am", "broadcast", "-a", packageName + "-Debug", "--ez", "enable", "true"], "exit").wait();
            var match = res.stdout.match(/result=(\d)+/);
            if (match) {
                port = match[0].substring(7);
            }
            else {
                port = 0;
            }
        }
        if ((0 < port) && (port < 65536)) {
            this.tcpForward(port, port);
            this.startDebuggerClient(port).wait();
            this.openDebuggerClient(this.defaultNodeInspectorUrl + "?port=" + port);
        }
        else {
            this.$logger.info("Cannot detect debug port.");
        }
    };
    AndroidDevice.prototype.detachDebugger = function (packageName) {
        var stopDebuggerCommand = this.composeCommand("shell am broadcast -a \"%s-Debug\" --ez enable false", packageName);
        this.$childProcess.exec(stopDebuggerCommand).wait();
    };
    AndroidDevice.prototype.startAppWithDebugger = function (packageFile, packageName) {
        var uninstallCommand = this.composeCommand("shell pm uninstall \"%s\"", packageName);
        this.$childProcess.exec(uninstallCommand).wait();
        var installCommand = this.composeCommand("install -r \"%s\"", packageFile);
        this.$childProcess.exec(installCommand).wait();
        var port = options["debug-port"];
        var packageDir = util.format(AndroidDevice.PACKAGE_EXTERNAL_DIR_TEMPLATE, packageName);
        var envDebugOutFullpath = packageDir + AndroidDevice.ENV_DEBUG_OUT_FILENAME;
        var clearDebugEnvironmentCommand = this.composeCommand('shell rm "%s"', envDebugOutFullpath);
        this.$childProcess.exec(clearDebugEnvironmentCommand).wait();
        this.startPackageOnDevice(packageName).wait();
        var dbgPort = this.startAndGetPort(packageName).wait();
        if (dbgPort > 0) {
            this.tcpForward(dbgPort, dbgPort);
            this.startDebuggerClient(dbgPort).wait();
            this.openDebuggerClient(this.defaultNodeInspectorUrl + "?port=" + dbgPort);
        }
    };
    AndroidDevice.prototype.debug = function (packageFile, packageName) {
        var _this = this;
        return (function () {
            if (options["get-port"]) {
                _this.printDebugPort(packageName);
            }
            else if (options["start"]) {
                _this.attachDebugger(packageName);
            }
            else if (options["stop"]) {
                _this.detachDebugger(packageName);
            }
            else if (options["debug-brk"]) {
                _this.startAppWithDebugger(packageFile, packageName);
            }
            else {
                _this.$logger.info("Should specify at least one option: debug-brk, start, stop, get-port.");
            }
        }).future()();
    };
    AndroidDevice.prototype.checkIfRunning = function (packageName) {
        var packageDir = util.format(AndroidDevice.PACKAGE_EXTERNAL_DIR_TEMPLATE, packageName);
        var envDebugOutFullpath = packageDir + AndroidDevice.ENV_DEBUG_OUT_FILENAME;
        var isRunning = this.checkIfFileExists(envDebugOutFullpath).wait();
        return isRunning;
    };
    AndroidDevice.prototype.checkIfFileExists = function (filename) {
        var _this = this;
        return (function () {
            var args = ["shell", "test", "-f", filename, "&&", "echo 'yes'", "||", "echo 'no'"];
            var res = _this.$childProcess.spawnFromEvent(_this.adb, args, "exit").wait();
            var exists = res.stdout.indexOf('yes') > -1;
            return exists;
        }).future()();
    };
    AndroidDevice.prototype.startAndGetPort = function (packageName) {
        var _this = this;
        return (function () {
            var port = -1;
            var packageDir = util.format(AndroidDevice.PACKAGE_EXTERNAL_DIR_TEMPLATE, packageName);
            var envDebugInFullpath = packageDir + AndroidDevice.ENV_DEBUG_IN_FILENAME;
            var clearDebugEnvironmentCommand = _this.composeCommand('shell rm "%s"', envDebugInFullpath);
            _this.$childProcess.exec(clearDebugEnvironmentCommand).wait();
            var isRunning = false;
            for (var i = 0; i < 60; i++) {
                helpers.sleep(1000);
                isRunning = _this.checkIfRunning(packageName);
                if (isRunning)
                    break;
            }
            if (isRunning) {
                var setEnvironmentCommand = _this.composeCommand('shell "cat /dev/null > %s"', envDebugInFullpath);
                _this.$childProcess.exec(setEnvironmentCommand).wait();
                for (var i = 0; i < 10; i++) {
                    helpers.sleep(1000);
                    var envDebugOutFullpath = packageDir + AndroidDevice.ENV_DEBUG_OUT_FILENAME;
                    var exists = _this.checkIfFileExists(envDebugOutFullpath).wait();
                    if (exists) {
                        var res = _this.$childProcess.spawnFromEvent(_this.adb, ["shell", "cat", envDebugOutFullpath], "exit").wait();
                        var match = res.stdout.match(/PORT=(\d)+/);
                        if (match) {
                            port = parseInt(match[0].substring(5), 10);
                            break;
                        }
                    }
                }
            }
            return port;
        }).future()();
    };
    AndroidDevice.prototype.ensureFullAccessPermissions = function (devicePath) {
        var command = this.composeCommand('shell chmod 0777 "%s"', devicePath);
        return this.$childProcess.exec(command);
    };
    AndroidDevice.prototype.pushFileOnDevice = function (localPath, devicePath) {
        var _this = this;
        return (function () {
            var rmCommand = _this.composeCommand('shell rm -r "%s"', devicePath);
            _this.$childProcess.exec(rmCommand).wait();
            if (_this.$fs.exists(localPath).wait()) {
                var isDirectory = _this.$fs.getFsStats(localPath).wait().isDirectory();
                var mkdirCommand = _this.composeCommand('shell mkdir -p "%s"', isDirectory ? devicePath : path.dirname(devicePath));
                _this.$childProcess.exec(mkdirCommand).wait();
                _this.pushFileOnDeviceCore(localPath, devicePath).wait();
            }
        }).future()();
    };
    AndroidDevice.prototype.pushFileOnDeviceCore = function (localPath, devicePath) {
        var _this = this;
        return (function () {
            var isDirectory = _this.$fs.getFsStats(localPath).wait().isDirectory();
            var pushFileCommand = _this.composeCommand('push "%s" "%s"', isDirectory ? path.join(localPath, ".") : localPath, devicePath);
            _this.$childProcess.exec(pushFileCommand).wait();
        }).future()();
    };
    AndroidDevice.prototype.sync = function (localToDevicePaths, appIdentifier, liveSyncUrl, syncOptions) {
        var _this = this;
        if (syncOptions === void 0) { syncOptions = {}; }
        return (function () {
            if (appIdentifier.isLiveSyncSupported(_this).wait()) {
                var liveSyncVersion = _this.getLiveSyncVersion(appIdentifier).wait();
                if (liveSyncVersion >= 2) {
                    _this.syncNewProtocol(localToDevicePaths, appIdentifier, liveSyncUrl, liveSyncVersion, syncOptions).wait();
                }
                else {
                    _this.syncOldProtocol(localToDevicePaths, appIdentifier, liveSyncUrl, syncOptions).wait();
                }
                _this.$logger.info("Successfully synced device with identifier '%s'", _this.getIdentifier());
            }
            else {
                _this.$errors.fail({ formatStr: appIdentifier.getLiveSyncNotSupportedError(_this), suppressCommandHelp: true });
            }
        }).future()();
    };
    AndroidDevice.prototype.runApplication = function (applicationId) {
        return this.startPackageOnDevice(applicationId);
    };
    AndroidDevice.prototype.listApplications = function () {
    };
    AndroidDevice.prototype.uninstallApplication = function (applicationId) {
        return (function () {
        }).future()();
    };
    AndroidDevice.prototype.listFiles = function (devicePath) {
        return (function () {
        }).future()();
    };
    AndroidDevice.prototype.getFile = function (deviceFilePath) {
        return (function () {
        }).future()();
    };
    AndroidDevice.prototype.putFile = function (localFilePath, deviceFilePath) {
        return (function () {
        }).future()();
    };
    AndroidDevice.prototype.getLiveSyncVersion = function (appIdentifier) {
        return this.sendBroadcastToDevice(AndroidDevice.CHECK_LIVESYNC_INTENT_NAME, { "app-id": appIdentifier.appIdentifier });
    };
    AndroidDevice.prototype.createLiveSyncCommandsFileOnDevice = function (liveSyncRoot, commands) {
        var _this = this;
        return (function () {
            var hostTmpDir = _this.getTempDir();
            var commandsFileHostPath = path.join(hostTmpDir, AndroidDevice.COMMANDS_FILE);
            var commandsFile = _this.$fs.createWriteStream(commandsFileHostPath);
            var fileWritten = _this.$fs.futureFromEvent(commandsFile, 'finish');
            _.each(commands, function (command) {
                commandsFile.write(command);
            });
            commandsFile.end();
            fileWritten.wait();
            var commandsFileDevicePath = _this.buildDevicePath(liveSyncRoot, AndroidDevice.COMMANDS_FILE);
            _this.pushFileOnDeviceCore(commandsFileHostPath, commandsFileDevicePath).wait();
            _this.ensureFullAccessPermissions(commandsFileDevicePath).wait();
        }).future()();
    };
    AndroidDevice.prototype.getTempDir = function () {
        temp.track();
        return temp.mkdirSync("ab-");
    };
    AndroidDevice.prototype.syncOldProtocol = function (localToDevicePaths, appIdentifier, liveSyncUrl, syncOptions) {
        var _this = this;
        if (syncOptions === void 0) { syncOptions = {}; }
        return (function () {
            _.each(localToDevicePaths, function (localToDevicePathData) {
                _this.pushFileOnDevice(localToDevicePathData.getLocalPath(), localToDevicePathData.getDevicePath()).wait();
            });
            if (!syncOptions.skipRefresh) {
                var changeLiveSyncUrlExtras = {
                    "liveSyncUrl": liveSyncUrl,
                    "app-id": appIdentifier.appIdentifier
                };
                _this.sendBroadcastToDevice(AndroidDevice.CHANGE_LIVESYNC_URL_INTENT_NAME, changeLiveSyncUrlExtras).wait();
                _this.sendBroadcastToDevice(AndroidDevice.REFRESH_WEB_VIEW_INTENT_NAME, { "app-id": appIdentifier.appIdentifier }).wait();
            }
        }).future()();
    };
    AndroidDevice.prototype.syncNewProtocol = function (localToDevicePaths, appIdentifier, liveSyncUrl, liveSyncVersion, syncOptions) {
        var _this = this;
        if (syncOptions === void 0) { syncOptions = {}; }
        return (function () {
            var liveSyncRoot = _this.getLiveSyncRoot(appIdentifier, liveSyncVersion);
            var dirs = Object.create(null);
            _.each(localToDevicePaths, function (localToDevicePathData) {
                var relativeToProjectBasePath = helpers.fromWindowsRelativePathToUnix(localToDevicePathData.getRelativeToProjectBasePath());
                var devicePath = _this.buildDevicePath(liveSyncRoot, relativeToProjectBasePath);
                _this.pushFileOnDevice(localToDevicePathData.getLocalPath(), devicePath).wait();
                if (liveSyncVersion === 2) {
                    var parts = relativeToProjectBasePath.split(AndroidDevice.DEVICE_PATH_SEPARATOR);
                    var currentPath = "";
                    _.each(parts, function (p) {
                        if (p !== "") {
                            currentPath = _this.buildDevicePath(currentPath, p);
                            if (!dirs[currentPath]) {
                                dirs[currentPath] = currentPath;
                                _this.ensureFullAccessPermissions(_this.buildDevicePath(liveSyncRoot, currentPath)).wait();
                            }
                        }
                    });
                }
            });
            _this.ensureFullAccessPermissions(liveSyncRoot).wait();
            if (!syncOptions.skipRefresh) {
                var commands = [];
                if (options.watch) {
                    commands = [
                        LiveSyncCommands.SyncFilesCommand(),
                        LiveSyncCommands.RefreshCurrentViewCommand()
                    ];
                }
                else {
                    commands = [
                        LiveSyncCommands.DeployProjectCommand(liveSyncUrl),
                        LiveSyncCommands.ReloadStartViewCommand()
                    ];
                }
                _this.createLiveSyncCommandsFileOnDevice(liveSyncRoot, commands).wait();
                _this.sendBroadcastToDevice(AndroidDevice.LIVESYNC_BROADCAST_NAME, { "app-id": appIdentifier.appIdentifier }).wait();
            }
        }).future()();
    };
    AndroidDevice.prototype.buildDevicePath = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i - 0] = arguments[_i];
        }
        return args.join(AndroidDevice.DEVICE_PATH_SEPARATOR);
    };
    AndroidDevice.prototype.openDeviceLogStream = function () {
        this.$logcatHelper.start(this.getIdentifier(), this.adb);
    };
    AndroidDevice.REFRESH_WEB_VIEW_INTENT_NAME = "com.telerik.RefreshWebView";
    AndroidDevice.CHANGE_LIVESYNC_URL_INTENT_NAME = "com.telerik.ChangeLiveSyncUrl";
    AndroidDevice.LIVESYNC_BROADCAST_NAME = "com.telerik.LiveSync";
    AndroidDevice.CHECK_LIVESYNC_INTENT_NAME = "com.telerik.IsLiveSyncSupported";
    AndroidDevice.ENV_DEBUG_IN_FILENAME = "envDebug.in";
    AndroidDevice.ENV_DEBUG_OUT_FILENAME = "envDebug.out";
    AndroidDevice.PACKAGE_EXTERNAL_DIR_TEMPLATE = "/sdcard/Android/data/%s/files/";
    AndroidDevice.DEVICE_TMP_DIR_FORMAT_V2 = "/data/local/tmp/12590FAA-5EDD-4B12-856D-F52A0A1599F2/%s";
    AndroidDevice.DEVICE_TMP_DIR_FORMAT_V3 = "/mnt/sdcard/Android/data/%s/files/12590FAA-5EDD-4B12-856D-F52A0A1599F2";
    AndroidDevice.COMMANDS_FILE = "telerik.livesync.commands";
    AndroidDevice.DEVICE_PATH_SEPARATOR = "/";
    return AndroidDevice;
})();
exports.AndroidDevice = AndroidDevice;
